#include "cifrasSubstituicao.hpp"
#include <cmath> // biblioteca da funo floor()

// ===//Funes Auxiliares (private)//===

int CifrasSubstituicao::mod(int numero, int base){
  int resto;

  // resto da diviso inteira, pode ser negativo
  resto = numero%base;  

  if(resto >= 0)
    return resto;
  else
    return (resto + base);
}



// Mximo divisor comum (no por recorrncia)
int CifrasSubstituicao::mdc(int a, int b) {
  int resto;
  while (b != 0) {
    resto = a % b;
    a = b;
    b = resto;
  }
  return a;
}

// Mximo divisor comum (por recorrncia)
int CifrasSubstituicao::mdcRec(int a, int b) {
  if (b == 0) { // caso de base
    return a;
  }
  else {
    return mdcRec(b,a%b);
  }
}
  

// predicado primos relativos ou nao
bool CifrasSubstituicao::primosRelativos(int a,int b){
  if(mdc(a,b)==1)
    return true;
  else
    return false;
}

// Inverso multiplicativo de a mod m
int CifrasSubstituicao::inversoMultiplicativo(int a, int m){

  int t0,t,temp,r,q,m1;

  m1=m;
  t0 = 0;
  t = 1;
  q = floor(m1/a);
  r = m1 - q*a;
  while(r > 0) {
    temp =(t0 - q*t) % m;
    t0 = t;
    t = temp;
    m1 = a;
    a = r;
    q = floor(m1/a);
    r = m1 - q*a;
  }
  if(a==1)
    return mod(t,m);
  else
    throw CifrasSubstituicao::naoInversoMultiplicativo();
}


// ===//(fim) Funes Auxiliares (private)//===


// ===//Funes de converso entre alfabetos//===

// ISO-8859-1 (latin 1) para 0--|A|
int CifrasSubstituicao::entradaParaAlfabeto(int n){
  if(n >= 97 && n <= 122) //[a-z]
    return n - 97;
  else if (n >= 224 && n <= 227) //[   ] 
    return n - 224 + 26;
  else if (n >= 231 && n <= 234)  //[   ]
    return n - 231 + 30;
  else if (n >= 236 && n <= 237)  //[ ]
    return n - 236 + 34;
  else if (n >= 242 && n <= 245)  //[   ]
    return n - 242 + 36;
  else if (n >= 249 && n <= 250) //[ ] 
    return n - 249 + 40;
  else if (n == 252)  //[]
    return n - 252 + 42;
  else
    throw CifrasSubstituicao::naoAlfa();
}

// 0--|A| para ISO-8859-1 (Latin 1)
int CifrasSubstituicao::alfabetoParaSaida(int n){
  
  if(n >= 0 && n <= 25) //[a-z]
    return n + 97;
  else if (n >=26 && n <= 29) //[   ]
    return n + 224 - 26;
  else if (n >= 30 && n <= 33) //[   ]
    return n + 231 - 30;
  else if (n >= 34 && n <= 35) //[ ]
    return n + 236 - 34;
  else if (n >= 36 && n <=39) //[   ]
    return n + 242 - 36;
  else if (n >= 40 && n <= 41) //[ ]
    return n + 249 - 40;
  else // if (n == 42) //[]
    return n + 252 - 42;
  
}

//===//(fim) Funes de converso entre alfabetos//===

//===Cifra Deslocamento Simples===//

/* Encriptar de um carcter, ISO-8859-1 */
int CifrasSubstituicao::encriptarDS(int m,int chave){ 
  int valorNoAlfabeto,encriptar,valorIso; 

  //obter o valor do carcter no Alfabeto 
  //verificar se o carcter pertence ao Alfabeto 
  try {
    valorNoAlfabeto=entradaParaAlfabeto(m);
    // Se no houve problemas
    encriptar=cifrarDS(valorNoAlfabeto,chave); 
    //obter o valor do caracter no ISO8859_1 
    valorIso=alfabetoParaSaida(encriptar);
    return valorIso;
  }
  catch (CifrasSubstituicao::naoAlfa) {
    // Os carcteres no considerados na cifra no so cifrados
    return m; 
  }
} 


/* Desencriptar de um carcter, ISO-8859-1 */
int CifrasSubstituicao::desencriptarDS(int c,int chave){ 
  int valorNoAlfabeto,desencriptar,valorIso,diferenca;

  //obter o valor do carcter no Alfabeto 
  //verificar se o carcter pertence ao Alfabeto 
  try {
    valorNoAlfabeto=entradaParaAlfabeto(c); 
    diferenca=decifrarDS(valorNoAlfabeto,chave);
    desencriptar=mod(diferenca,compAlfa); 
    valorIso=alfabetoParaSaida(desencriptar); 
    return valorIso; 
  }
  catch (CifrasSubstituicao::naoAlfa) {
    // Os carcteres no considerados na cifra no so decifrados
    return c;
  }
}



// Cifra de Deslocamento Simples
int CifrasSubstituicao::cifrarDS(int m,int c){
  
  return((m + c) % compAlfa);
  
}


//  Cifra de Deslocamento Simples (funo inversa)
int CifrasSubstituicao::decifrarDS(int e,int c){
  
  return(mod((e - c),compAlfa));
  
}

//===(fim)Cifra Deslocamento Simples===//

//===Cifra Deslocamento Linear===//

// Encriptar carcter (deslocamento linear)
int CifrasSubstituicao::encriptarDL(int m,int chave1,int chave2){

  int valornoalfabeto,encriptar,valoriso;

  try {
    //obter o valor do carcter no Alfabeto
    valornoalfabeto=entradaParaAlfabeto(m);
    // No caso em que est tudo bem
    // aplicar o algoritmo para encriptar
    encriptar=cifrarDL(valornoalfabeto,chave1,chave2);
    //obter o valor do carcter no ISO8859 
    //retornar o valor
    valoriso = alfabetoParaSaida(encriptar);
    return valoriso;
  }
  catch (CifrasSubstituicao::naoAlfa) {
    // Os carcteres no considerados na cifra no so decifrados
    return m;
  }
}

// Desencriptar caracteres (deslocamento linear)
int  CifrasSubstituicao::desencriptarDL(int c,int chave1,int chave2){

  int inverso,desencriptar,valorISO,valorNoAlfabeto,resultado;
  
  try {
    // obter o valor do carcter no Alfabeto
    valorNoAlfabeto=entradaParaAlfabeto(c); 
    // caso no haja erros, o carcter pertence ao alfabeto 
    // aplicar o algoritmo para desencriptar
    resultado=decifrarDL(valorNoAlfabeto,chave1,chave2);
    desencriptar=mod(resultado,compAlfa);
    // obter o valor do carcter no ISO8859 
    // retornar o valor
    valorISO = alfabetoParaSaida(desencriptar);
    return valorISO;
  }
  catch (CifrasSubstituicao::naoAlfa) { // excepo
    // Os carcteres no considerados na cifra no so decifrados
    return c;
  }
}  


// Cifra de Deslocamento Linear (cifrar)
int CifrasSubstituicao::cifrarDL(int m,int chave1,int chave2){

  return((chave1*m + chave2) % compAlfa);
}


// Cifra de Deslocamento Linear (decifrar)
int CifrasSubstituicao::decifrarDL(int c,int chave1,int chave2){
  
  int inverso;

  //chamar a funo inversoMultiplicativo()
  try {
    inverso=inversoMultiplicativo(chave1,compAlfa);
    return(inverso*(c - chave2) % compAlfa);
  }
  catch (CifrasSubstituicao::naoInversoMultiplicativo) {
    // em caso de erro, opta-se por no desencriptar
    return(c);
  }
}
  
//===(fim)Cifra Deslocamento Linear===//

