/*
 * Cifras Clssicas de Substituio
 *
 * Cifra de Deslocamento Simples
 * Cifra de Deslocamento Linear
 */

#ifndef CIFRASSUBSTITUICAO
#define CIFRASSUBSTITUICAO

class CifrasSubstituicao {
private:
  /* Comprimento do Alfabeto */
  int compAlfa = 26; 
  /* Cdigo de erro para nmeros que no so primos entre si */
  struct erroPrimosRelativos {};
  /*  Cdigo de erro para caracteres fora do Alfabeto */
  struct naoAlfa {};
  struct naoInversoMultiplicativo {};

  /*
   * Mtodo mdulo
   * --> numero : inteiro 
   * --> base : inteiro 
   * <-- resto : resto (>0) da diviso inteira de numero por base
   *

   * Ajusta o valor do resto para o caso em que o nmero de entrada 
   * negativo
   */
  int mod(int numero, int base);
  
  /*
   * mximo divisor comum entre dois inteiros
   * usa o Algoritmo de Euclides
   * --> a,b : inteiros, a > b
   * <-- mdc(a,b): 
   */
  int mdc(int a, int b);

  // A mesma funo mas implementada por recorrncia
  int mdcRec(int a, int b);
  
  /*
   * Verifica se dois inteiros so primos relativos
   * a,b so primos relativos se mdc(a,b) = 1;
   * -->  a,b : inteiros, a > b
   * <-- v : v=true, so primos relativos, false, caso contrrio
   */
  bool primosRelativos(int a,int b);

  /*
   * Clculo do inverso multiplicativo, mdulo m
   * ---> a,m : inteiros
   * <--- im : a^-1 : inverso multiplicativo mdulo m, 
   *                  ou -1 caso no exista inverso multiplicativo
   */
  int inversoMultiplicativo(int a, int m);

public:

  /* 
   * Mtodo que converte a codificao ASCII para Alfabeto-A 
   * --> n, codificao do carcter lido
   * <-- m, valor ajustado para a gama 0<= m < compAlfa
   *
   * Erro: caso de um carcter no reconhecido
  */
  int entradaParaAlfabeto(int n);

  /*
   * Mtodo que converte do Alfabeto-A para Codificao ASCII
   * --> m, 0<= m < compAlfa
   * <-- n, codificao do carcter correspondente
   */
  int alfabetoParaSaida(int n);

  /*
   * =====================================
   * === Cifra de Deslocamento Simples ===
   * =====================================
   */
  
  /*
   * Encriptar atravs da cifra Deslocamento Simples
   * --> m, carcter a encriptar, codificao ISO-8859-1
   *     chave, 0<= m < compAlfa
   * <-- c, carcter cifrado, codificao ISO-8859-1
   */
  int encriptarDS(int m,int chave);
  
  /*
   * Desencriptar atravs da cifra Deslocamento Simples
   * --> c, carcter a desencriptar, codificao ISO-8859-1
   *     chave, 0<= m < compAlfa
   * <-- n,  carcter desencriptado, codificao ISO-8859-1
   */
  int desencriptarDS(int c,int chave);

  /*
   * Cifra de Deslocamento Simples
   * 
   * --> m, 0<= m < compAlfa, nmero natural a encriptar
   *     chave, 0<= chave < compAlfa, chave para encriptar
   * <-- c, 0<= m < compAlfa, nmero natural encriptado   
   */
  int cifrarDS(int m,int c);


  /*
   * Cifra de Deslocamento Simples (funo inversa)
   *    
   * --> e, 0<= m < compAlfa, nmero natural a desencriptar
   *     c, 0<= c < compAlfa, chave para encriptar
   * <-- d, 0<= c < compAlfa, nmero natural desencriptado
   */
  int decifrarDS(int e,int c);

  /*
   * ====================================
   * === Cifra de Deslocamento Linear ===
   * ====================================
   */
  /*
   * Encriptar atravs da cifra Deslocamento Linear, |A| = ISO-8859-1
   * --> m: carcter a encriptar
   *     chave1, chave2: chave1 tem de ser um primo relativo com |A|
   * <-- c, carcter encriptado
   */
  int encriptarDL(int m,int chave1,int chave2);

  /*
   * Desencriptar atravs da cifra Deslocamento Simples, |A| = ISO-8859-1
   * --> c: carcter a desencriptar
   *     chave1, chave2:
   * <-- n: 
   */
  int desencriptarDL(int c,int chave1,int chave2);

  /*
   * Cifra de Deslocamento Linear (cifrar)
   * 
   * --> m: inteiro, 0<= m < compAlfa
   *     chave1,chave2: chave1 tem de ser invertvel, mod |A|
   * <-- c: c=chave1*m+chave2 mod 26
   */
  int cifrarDL(int m,int chave1,int chave2);

  /*
   * Cifra de Deslocamento Linear (decifrar)
   * Excepo: no existe inverso multiplicativo
   *   
   * --> c : inteiro, 0<= m < compAlfa
   *     chave1: inverso multiplicativo de chave1 (mod |A|)
   *     chave2: 
   * <-- d,  d=chave1^-1*(m-chave2) mod 26
   *     Excepo: naoInversoMultiplicativo
   */
  int decifrarDL(int c,int invchave1,int chave2);  
};

#endif
